@@ -296,7 +296,7 @@ int BasicMBRData::ReadLogicalParts(uint64_t extendedStart, int partNum) {
          // the logical partition when this is the case....
          ebrType = ebr.partitions[0].partitionType;
          if ((ebrType == 0x05) || (ebrType == 0x0f) || (ebrType == 0x85)) {
-            cout << "EBR describes a logical partition!\n";
+            cout << "EBR points to an EBR!\n";
             offset = extendedStart + ebr.partitions[0].firstLBA;
          } else {
             // Copy over the basic data....
@@ -671,7 +671,7 @@ int BasicMBRData::LBAtoCHS(uint64_t lba, uint8_t * chs) {
          done = 1;
       } // if
       // If LBA value is too large for CHS, max out CHS values....
-      if ((!done) && (lba >= (numHeads * numSecspTrack * MAX_CYLINDERS))) {
+      if ((!done) && (lba >= ((uint64_t) numHeads * numSecspTrack * MAX_CYLINDERS))) {
          chs[0] = 254;
          chs[1] = chs[2] = 255;
          done = 1;
@@ -726,7 +726,7 @@ int BasicMBRData::FindOverlaps(void) {
    if (numEE > 1)
       cout << "\nCaution: More than one 0xEE MBR partition found. This can cause problems\n"
            << "in some OSes.\n";
-   if (!ProtectiveOnOne)
+   if (!ProtectiveOnOne && (numEE > 0))
       cout << "\nWarning: 0xEE partition doesn't start on sector 1. This can cause "
            << "problems\nin some OSes.\n";
 
@@ -1223,7 +1223,7 @@ void BasicMBRData::MaximizePrimaries() {
 // Remove primary partitions in excess of 4, starting with the later ones,
 // in terms of the array location....
 void BasicMBRData::TrimPrimaries(void) {
-   int numToDelete, i = MAX_MBR_PARTS;
+   int numToDelete, i = MAX_MBR_PARTS - 1;
 
    numToDelete = NumPrimaries() - 4;
    while ((numToDelete > 0) && (i >= 0)) {
