@@ -34,6 +34,24 @@ using namespace std;
 // Reads a string from stdin, returning it as a C++-style string.
 // Note that the returned string will NOT include the carriage return
 // entered by the user.
+#ifdef EFI
+extern int __sscanf( const char * str , const char * format , ... ) ;
+string ReadString(void) {
+   string inString;
+   char efiString[256];
+   int stringLength;
+
+   if (fgets(efiString, 255, stdin) != NULL) {
+      stringLength = strlen(efiString);
+      if ((stringLength > 0) && (efiString[stringLength - 1] == '\n'))
+          efiString[stringLength - 1] = '\0';
+      inString = efiString;
+   } else {
+      inString = "";
+   }
+   return inString;
+} // ReadString()
+#else
 string ReadString(void) {
    string inString;
 
@@ -42,6 +60,7 @@ string ReadString(void) {
       exit(5);
    return inString;
 } // ReadString()
+#endif
 
 // Get a numeric value from the user, between low and high (inclusive).
 // Keeps looping until the user enters a value within that range.
@@ -77,8 +96,11 @@ int GetNumber(int low, int high, int def, const string & prompt) {
 char GetYN(void) {
    char response;
    string line;
+   bool again = 0 ;
 
    do {
+      if ( again ) { cout << "Your option? " ; }
+      again = 1 ;
       cout << "(Y/N): ";
       line = ReadString();
       response = toupper(line[0]);
@@ -129,13 +151,13 @@ uint64_t GetSectorNum(uint64_t low, uint64_t high, uint64_t def, uint64_t sSize,
 uint64_t IeeeToInt(string inValue, uint64_t sSize, uint64_t low, uint64_t high, uint64_t def) {
    uint64_t response = def, bytesPerUnit = 1, mult = 1, divide = 1;
    size_t foundAt = 0;
-   char suffix, plusFlag = ' ';
+   char suffix = ' ', plusFlag = ' ';
    string suffixes = "KMGTPE";
    int badInput = 0; // flag bad input; once this goes to 1, other values are irrelevant
 
    if (sSize == 0) {
       sSize = SECTOR_SIZE;
-      cerr << "Bug: Sector size invalid in SIToInt()!\n";
+      cerr << "Bug: Sector size invalid in IeeeToInt()!\n";
    } // if
 
    // Remove leading spaces, if present
@@ -214,30 +236,39 @@ uint64_t IeeeToInt(string inValue, uint64_t sSize, uint64_t low, uint64_t high,
 // Takes a size and converts this to a size in IEEE-1541-2002 units (KiB, MiB,
 // GiB, TiB, PiB, or EiB), returned in C++ string form. The size is either in
 // units of the sector size or, if that parameter is omitted, in bytes.
-// (sectorSize defaults to 1).
+// (sectorSize defaults to 1). Note that this function uses peculiar
+// manual computation of decimal value rather than simply setting
+// theValue.precision() because this isn't possible using the available
+// EFI library.
 string BytesToIeee(uint64_t size, uint32_t sectorSize) {
-   float sizeInIeee;
-   unsigned int index = 0;
+   uint64_t sizeInIeee;
+   uint64_t previousIeee;
+   float decimalIeee;
+   uint64_t index = 0;
    string units, prefixes = " KMGTPEZ";
    ostringstream theValue;
 
-   sizeInIeee = size * (float) sectorSize;
-   while ((sizeInIeee > 1024.0) && (index < (prefixes.length() - 1))) {
+   sizeInIeee = previousIeee = size * (uint64_t) sectorSize;
+   while ((sizeInIeee > 1024) && (index < (prefixes.length() - 1))) {
       index++;
-      sizeInIeee /= 1024.0;
+      previousIeee = sizeInIeee;
+      sizeInIeee /= 1024;
    } // while
-   theValue.setf(ios::fixed);
    if (prefixes[index] == ' ') {
-      units = " bytes";
-      theValue.precision(0);
+      theValue << sizeInIeee << " bytes";
    } else {
       units = "  iB";
       units[1] = prefixes[index];
-      theValue.precision(1);
+      decimalIeee = ((float) previousIeee -
+                     ((float) sizeInIeee * 1024.0) + 51.2) / 102.4;
+      if (decimalIeee >= 10.0) {
+         decimalIeee = 0.0;
+         sizeInIeee++;
+      }
+      theValue << sizeInIeee << "." << (uint32_t) decimalIeee << units;
    } // if/else
-   theValue << sizeInIeee << units;
    return theValue.str();
-} // BlocksToIeee()
+} // BytesToIeee()
 
 // Converts two consecutive characters in the input string into a
 // number, interpreting the string as a hexadecimal number, starting
@@ -316,11 +347,11 @@ void ReverseBytes(void* theValue, int numBytes) {
 void WinWarning(void) {
    #ifdef _WIN32
    cout << "\a************************************************************************\n"
-   << "Most versions of Windows cannot boot from a GPT disk, and most varieties\n"
-   << "prior to Vista cannot read GPT disks. Therefore, you should exit now\n"
-   << "unless you understand the implications of converting MBR to GPT or creating\n"
-   << "a new GPT disk layout!\n"
-   << "************************************************************************\n\n";
+        << "Most versions of Windows cannot boot from a GPT disk except on a UEFI-based\n"
+        << "computer, and most varieties prior to Vista cannot read GPT disks. Therefore,\n"
+        << "you should exit now unless you understand the implications of converting MBR\n"
+        << "to GPT or creating a new GPT disk layout!\n"
+        << "************************************************************************\n\n";
    cout << "Are you SURE you want to continue? ";
    if (GetYN() != 'Y')
       exit(0);
